
/*
 * Copyright Attribution.  Each Licensee Product shall display, in a 
 * user-visible location such as a start-up screen or an "About" box, 
 * and in a message to be displayed on-screen at the time Licensee 
 * Products are installed on an end-user system, the following legend, 
 * which shall be displayed for at least two (2) seconds.
 * 
 * 	"PXE Software Copyright (C) 1999  Intel Corporation"
 */

#ifndef _CDHCP_CLASS_H
#define _CDHCP_CLASS_H

#include "pxemisc.h"

#define MAGIC_COOKIE 0x63538263 // Network order
#define MINPKTSIZE		576
#define HDRSIZE			236
#define COOKIESIZE		4
// 4 for the cookie, 1 for terminating 0xFF
#define MINOPTUSED		5

struct dhcpheader
{
	BYTE	opcode,			/* operation code */
			hardware,		/* hardware type */
			hardlen,		/* length of hardware address */
			gatehops;		/* gateways hops */
	ULONG	ident;			/* transaction identification */
	USHORT	seconds,		/* seconds elapsed since boot began */
			flags;			/* flags */
	ULONG	cip,			/* client IP address */
			yip,			/* your IP address */
			sip,			/* server IP address */
			gip;			/* gateway IP address */
	BYTE	caddr[16],		/* client hardware address */
			sname[64],		/* server name */
			bootfile[128];	/* bootfile name */
	ULONG	magic;			/* magic cookie */
};
		
class CDhcp
{
	private:
		// Fixed header portion of the DHCP packet
		struct dhcpheader m_Hdr;

		// Array of ptrs to BYTE arrays.  The index corresponds to the option number.
		BYTE* m_pucOpts[256];

		// Array of ptrs to BYTE arrays.  Each element is a option type 43 encapsulated
		// option.  The index corresponds to the option number.
		BYTE* m_pucOpts43[256];

		// Total number of bytes available for options including the magic cookie.  This value
		// plus 236 is the total size of the DHCP packet.
		int m_OptionSize;

		// Total number of bytes already used for options.  The minimum value is 4 for the
		// four bytes in the magic cookie.
		int m_OptionsUsed;

		// Pointer to extended PXECLient Tag
		BYTE* m_pExtendedPxeClientTag;

		BYTE* m_pOptTag93;
		BYTE* m_pOptTag94;

		BOOL m_bOptTag93Same;
		BOOL m_bOptTag94Same;

	public:
		// OBJECT CONSTRUCTORS
		// Create a new CDhcp object of the minimum size initialized to zero
		CDhcp();

		// Create a new CDhcp object of size p_nSize initialized to zero
		CDhcp(USHORT p_usSize);

		// Create a new CDhcp object of size p_nSize.  p_pucData is a byte
		// array containing a DHCP packet.
		CDhcp(const BYTE* p_pucData, const USHORT p_usSize);

		// Create a new CDhcp  object the same size as p_pucDhcpPacket and copy
		// p_pucDhcpPacket to the new object.
		CDhcp(const CDhcp& p_pucDhcpPacket);

		//Object Destructor
		~CDhcp();

		// OPERATORS
		// Assignment
		const CDhcp& CDhcp::operator =(const CDhcp& p_Pkt);

		void Initialize();

		// STATE QUERIES
		// Each of these returns the value in the specified field.  All multi-byte values are
		// returned in network order.
		BYTE Opcode();
		BYTE Hardware();
		BYTE HardwareLength();
		BYTE GatewayHops();
		ULONG TransactionID();
		USHORT Seconds();
		USHORT Flags();
		ULONG ClientIP();
		ULONG YourIP();
		ULONG ServerIP();
		ULONG GatewayIP();
		BYTE HardwareAddress(BYTE *p_ucData, int p_nSize);
		BYTE ServerName(BYTE *p_ucName, int p_nSize);
		BYTE BootFileName(BYTE *p_ucName, int p_nSize);
		ULONG MagicCookie();

		// Return the current length of the packet.
		USHORT PacketLength();

		//Return the maximum number of bytes the packet is allowed to hold.
		USHORT PacketSize();

		//Return number of bytes allocated for the option portion of the packet including
		//the magic cookie.
		USHORT OptionsSize();

		//Return the number of bytes currently used in the option portion of the packet,
		// including the magic cookie.
		USHORT OptionsLength();

		//Return the number of bytes remaining in the options portion of the packet.
		USHORT OptionsAvailableLength();

		// Copy contents of the CDhcp object to p_pucData as a BYTE array.  
		// p_nSize contains number of BYTES available in p_pbData.
		// Return TRUE if successful. Return FALSE if p_nSize is too small to contain
		// the packet.
		BOOL GetPacket(BYTE* p_pucData, int& p_nSize);

		//Copy only the data for the option specified by p_ucTag from the option portion of the packet
		//to p_pucData.  p_nSize indicates the number of bytes available.  Return the number of bytes
		//copied.  If p_nSize is too small return 0.  If the option is not found return 0.
		//The length of the data for the option can be determined by setting p_pucData to NULL
		// and p_nSize to 0.  The number of bytes in the option data is returned.  If the option
		//is not found 0 is returned.
		//If there are multiple options of the same type, the data for the first one found is
		//returned.
		int GetOption(BYTE p_ucTag, BYTE* p_pucData, int p_nSize);

		//Copy only the data for the 43 sub-option specified by p_ucTag from the option portion of
		//the packet to p_pucData.  p_nSize indicates the number of bytes available.  Return the
		//number of bytes copied.  If p_nSize is too small return 0.  If the option is not found
		//return 0.
		//The length of the data for the option can be determined by setting p_pucData to NULL
		//and p_nSize to 0.  The number of bytes in the option data is returned.  If the option
		//is not found 0 is returned.
		//All 43 options will be searched for the sub-option.  If there are multiple sub-options
		//of the same type, the data for the first one found is returned.
		int Get43Option(BYTE p_ucTag, BYTE* p_pucData, int p_nSize);

		//STATE CONSTRUCTORS
		//Each of these functions sets the specified field to the value provided.  The caller
		//must perform any conversions to network order.
		void Opcode(BYTE p_ucCode);
		void Hardware(BYTE p_ucHw);
		void HardwareLength(BYTE p_ucHwLen);
		void GatewayHops(BYTE p_ucHops);
		void TransactionID(ULONG p_ulId);
		void Seconds(USHORT p_usSecs);
		void Flags(USHORT p_usFlags);
		void ClientIP(ULONG p_ulCiaddr);
		void YourIP(ULONG p_ulYiaddr);
		void ServerIP(ULONG p_ulSiaddr);
		void GatewayIP(ULONG p_ulGiaddr);
		void SetHardwareAddress(BYTE* p_ucData, int p_nSize);
		void SetServerName(BYTE *p_ucName, int p_nSize);
		void SetBootFileName(BYTE *p_ucName, int p_nSize);
		void MagicCookie(ULONG p_ulCookie);

		//Insert the option specified by p_ucTag.  Return TRUE if successful.  Return FALSE if there
		//is not enough room left in the packet.
		BOOL AddOption(BYTE p_cTag, BYTE* p_pucData, int nSize);

		//Delete the option specified by p_ucTag if the option has been previously defined.
		void DelOption(BYTE p_ucTag);

		//Add an encapsulated vendor specific information into a type 43 vendor specified information
		//option.  Insert the sub-option specified by p_ucTag into a 43 option.  Return TRUE if
		//successful.  Return FALSE if there is not enough room left in the packet.
		//Encapsulated vendor specific information will be packed into type 43 options with a
		//maximum length of 255.  The order the options are inserted may not be the same order the
		//options are placed into the type 43 options.
		//Multiple type 43 options will be created if the total length of the options exceeds 255.
		BOOL Add43Option(BYTE p_cTag, BYTE* p_pucData, int nSize);

		//Delete the option specified by p_ucTag if the option has been previously defined.
		void Del43Option(BYTE p_ucTag);

		//Set the available length of the options portion of the packet.  If  p_usSize is larger
		//then more space is allocated as needed.  Previous option data is preserved.  If p_usSize
		//is smaller and the current the number of bytes in the current data exceeds the new
		//p_usSize, the packet remains unchanged and FALSE is returned.  FALSE is also returned
		//if p_nSize is smaller then 312 the minimum size allowed for the options in a DHCP
		//packet (RFC 2131).
		//Returns TRUE if successful otherwise returns FALSE.
		BOOL SetSize(USHORT p_usSize);

		void checkTags(BYTE* pExtendedTag, BYTE* pOldTypeTag, int len);
		BOOL GetOldStyleOpt93(CString *pcs, BOOL* bSame);
		BOOL GetOldStyleOpt94(CString *pcs, BOOL* pbSame);
		BOOL GetExtendedPxeClientTag(CString *pcs);



		// HELPER FUNCTIONS
		private:
			BOOL CDhcp::GetNextOption(const BYTE* p_pucOpts, USHORT& usIdx, USHORT usOptlen);
			// Parse option and store in option array, either m_pucOpts or m_pucOpts43.
			void CDhcp::StoreOption(const BYTE* pucOpts, USHORT usIdx, USHORT usOptLen);
			BOOL CDhcp::HandleNewClassIdentifier(const BYTE *pucOpts, const USHORT usIdx, USHORT ucLen);


};

#endif
